@@ -31,34 +31,29 @@ extern "C"
 
 #define ZYPP_USE_RESOLVER_INTERNALS
 
-#include "zypp/base/String.h"
-#include "zypp/Product.h"
-#include "zypp/Capability.h"
-#include "zypp/ResStatus.h"
-#include "zypp/VendorAttr.h"
-#include "zypp/base/LogTools.h"
-#include "zypp/base/String.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/Algorithm.h"
-#include "zypp/ResPool.h"
-#include "zypp/ResFilters.h"
-#include "zypp/ZConfig.h"
-#include "zypp/sat/Pool.h"
-#include "zypp/sat/WhatProvides.h"
-#include "zypp/sat/WhatObsoletes.h"
-#include "zypp/solver/detail/Resolver.h"
-#include "zypp/solver/detail/SATResolver.h"
-#include "zypp/solver/detail/ProblemSolutionCombi.h"
-#include "zypp/solver/detail/ProblemSolutionIgnore.h"
-#include "zypp/solver/detail/SolverQueueItemInstall.h"
-#include "zypp/solver/detail/SolverQueueItemDelete.h"
-#include "zypp/solver/detail/SystemCheck.h"
-#include "zypp/solver/detail/SolutionAction.h"
-#include "zypp/solver/detail/SolverQueueItem.h"
-#include "zypp/sat/Transaction.h"
-#include "zypp/sat/Queue.h"
+#include <zypp/base/LogTools.h>
+#include <zypp/base/Gettext.h>
+#include <zypp/base/Algorithm.h>
 
-#include "zypp/sat/detail/PoolImpl.h"
+#include <zypp/ZConfig.h>
+#include <zypp/Product.h>
+#include <zypp/AutoDispose.h>
+#include <zypp/sat/WhatProvides.h>
+#include <zypp/sat/WhatObsoletes.h>
+#include <zypp/sat/detail/PoolImpl.h>
+
+#include <zypp/solver/detail/Resolver.h>
+#include <zypp/solver/detail/SATResolver.h>
+
+#include <zypp/solver/detail/ProblemSolutionCombi.h>
+#include <zypp/solver/detail/ProblemSolutionIgnore.h>
+#include <zypp/solver/detail/SolverQueueItemInstall.h>
+#include <zypp/solver/detail/SolverQueueItemDelete.h>
+#include <zypp/solver/detail/SystemCheck.h>
+#include <zypp/solver/detail/SolutionAction.h>
+#include <zypp/solver/detail/SolverQueueItem.h>
+
+using std::endl;
 
 #define XDEBUG(x) do { if (base::logger::isExcessive()) XXX << x << std::endl;} while (0)
 
@@ -68,25 +63,6 @@ extern "C"
 /////////////////////////////////////////////////////////////////////////
 namespace zypp
 { ///////////////////////////////////////////////////////////////////////
-
-  /////////////////////////////////////////////////////////////////////////
-  namespace env
-  {
-    inline bool HACKENV( const char * var_r, bool default_r )
-    {
-      bool ret = default_r;
-      const char * val = ::getenv( var_r );
-      if ( val )
-      {
-	ret = str::strToBool( val, default_r );
-	if ( ret != default_r )
-	  INT << "HACKENV " << var_r << " = " << ret << endl;
-      }
-      return ret;
-    }
-  } // namespace env
-  /////////////////////////////////////////////////////////////////////////
-
   ///////////////////////////////////////////////////////////////////////
   namespace solver
   { /////////////////////////////////////////////////////////////////////
@@ -94,7 +70,101 @@ namespace zypp
     namespace detail
     { ///////////////////////////////////////////////////////////////////
 
-using namespace std;
+      ///////////////////////////////////////////////////////////////////////
+      namespace
+      {
+	inline void solverSetFocus( sat::detail::CSolver & satSolver_r, const ResolverFocus & focus_r )
+	{
+	  switch ( focus_r )
+	  {
+	    case ResolverFocus::Default:	// fallthrough to Job
+	    case ResolverFocus::Job:
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_INSTALLED, 0 );
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_BEST,      0 );
+	      break;
+	    case ResolverFocus::Installed:
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_INSTALLED, 1 );
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_BEST,      0 );
+	      break;
+	    case ResolverFocus::Update:
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_INSTALLED, 0 );
+	      solver_set_flag( &satSolver_r, SOLVER_FLAG_FOCUS_BEST,      1 );
+	      break;
+	  }
+	}
+
+	/** Helper collecting pseudo installed items from the pool.
+	 * \todo: pseudoItems are cachable as long as pool content does not change
+	 */
+	inline sat::Queue collectPseudoInstalled( const ResPool & pool_r )
+	{
+	  sat::Queue ret;
+	  for ( const PoolItem & pi : pool_r )
+	    if ( traits::isPseudoInstalled( pi.kind() ) ) ret.push( pi.id() );
+	  return ret;
+	}
+
+	/** Copy back new \ref WeakValue to \ref PoolItem after solving.
+	 * On the fly collect orphaned items (cached by the solver for the UI)
+	 */
+	inline void solverCopyBackWeak( sat::detail::CSolver & satSolver_r, PoolItemList & orphanedItems_r )
+	{
+	  // NOTE: assert all items weak stati are reset (resetWeak was called)
+	  {
+	    sat::Queue recommendations;
+	    sat::Queue suggestions;
+	    ::solver_get_recommendations( &satSolver_r, recommendations, suggestions, 0 );
+	    for ( sat::Queue::size_type i = 0; i < recommendations.size(); ++i )
+	      PoolItem(sat::Solvable(recommendations[i])).status().setRecommended( true );
+	    for ( sat::Queue::size_type i = 0; i < suggestions.size(); ++i )
+	      PoolItem(sat::Solvable(suggestions[i])).status().setSuggested( true );
+	  }
+	  {
+	    orphanedItems_r.clear();	// cached on the fly
+	    sat::Queue orphaned;
+	    ::solver_get_orphaned( &satSolver_r, orphaned );
+	    for ( sat::Queue::size_type i = 0; i < orphaned.size(); ++i )
+	    {
+	      PoolItem pi { sat::Solvable(orphaned[i]) };
+	      pi.status().setOrphaned( true );
+	      orphanedItems_r.push_back( pi );
+	    }
+	  }
+	  {
+	    sat::Queue unneeded;
+	    ::solver_get_unneeded( &satSolver_r, unneeded, 1 );
+	    for ( sat::Queue::size_type i = 0; i < unneeded.size(); ++i )
+	      PoolItem(sat::Solvable(unneeded[i])).status().setUnneeded( true );
+	  }
+	}
+
+	/** Copy back new \ref ValidateValue to \ref PoolItem after solving. */
+	inline void solverCopyBackValidate( sat::detail::CSolver & satSolver_r, const ResPool & pool_r )
+	{
+	  sat::Queue pseudoItems { collectPseudoInstalled( pool_r ) };
+	  if ( ! pseudoItems.empty() )
+	  {
+	    sat::Queue pseudoFlags;
+	    ::solver_trivial_installable( &satSolver_r, pseudoItems, pseudoFlags );
+
+	    for ( sat::Queue::size_type i = 0; i < pseudoItems.size(); ++i )
+	    {
+	      PoolItem pi { sat::Solvable(pseudoItems[i]) };
+	      switch ( pseudoFlags[i] )
+	      {
+		case 0:  pi.status().setBroken(); break;
+		case 1:  pi.status().setSatisfied(); break;
+		case -1: pi.status().setNonRelevant(); break;
+		default: pi.status().setUndetermined(); break;
+	      }
+	    }
+	  }
+	}
+
+      } //namespace
+      ///////////////////////////////////////////////////////////////////////
+
+
 
 IMPL_PTR_TYPE(SATResolver);
 
@@ -110,6 +180,50 @@ int vendorCheck( sat::detail::CPool *pool, Solvable *solvable1, Solvable *solvab
                                             IdString(solvable2->vendor) ) ? 0 : 1;
 }
 
+/** ResPool helper to compute the initial status of Patches etc.
+ * An empty solver run (no jobs) just to compute the initial status
+ * of pseudo installed items (patches).
+ */
+void establish( sat::Queue & pseudoItems_r, sat::Queue & pseudoFlags_r )
+{
+  pseudoItems_r = collectPseudoInstalled( ResPool::instance() );
+  if ( ! pseudoItems_r.empty() )
+  {
+    MIL << "Establish..." << endl;
+    sat::detail::CPool * cPool { sat::Pool::instance().get() };
+    ::pool_set_custom_vendorcheck( cPool, &vendorCheck );
+
+    sat::Queue jobQueue;
+    // Add rules for parallel installable resolvables with different versions
+    for ( const sat::Solvable & solv : sat::Pool::instance().multiversion() )
+    {
+      jobQueue.push( SOLVER_NOOBSOLETES | SOLVER_SOLVABLE );
+      jobQueue.push( solv.id() );
+    }
+
+    AutoDispose<sat::detail::CSolver*> cSolver { ::solver_create( cPool ), ::solver_free };
+    sat::Pool::instance().prepare();
+    if ( ::solver_solve( cSolver, jobQueue ) != 0 )
+      INT << "How can establish fail?" << endl;
+
+    ::solver_trivial_installable( cSolver, pseudoItems_r, pseudoFlags_r );
+
+    for ( sat::Queue::size_type i = 0; i < pseudoItems_r.size(); ++i )
+    {
+      PoolItem pi { sat::Solvable(pseudoItems_r[i]) };
+      switch ( pseudoFlags_r[i] )
+      {
+	case 0:  pi.status().setBroken(); break;
+	case 1:  pi.status().setSatisfied(); break;
+	case -1: pi.status().setNonRelevant(); break;
+	default: pi.status().setUndetermined(); break;
+      }
+    }
+    MIL << "Establish DONE" << endl;
+  }
+  else
+    MIL << "Establish not needed." << endl;
+}
 
 inline std::string itemToString( const PoolItem & item )
 {
@@ -127,14 +241,6 @@ inline std::string itemToString( const PoolItem & item )
   return ret;
 }
 
-inline PoolItem getPoolItem( Id id_r )
-{
-  PoolItem ret( (sat::Solvable( id_r )) );
-  if ( !ret && id_r )
-    INT << "id " << id_r << " not found in ZYPP pool." << endl;
-  return ret;
-}
-
 //---------------------------------------------------------------------------
 
 std::ostream &
@@ -162,9 +268,9 @@ SATResolver::dumpOn( std::ostream & os ) const
 	OUTS( DUP_ALLOW_NAMECHANGE );
 	OUTS( KEEP_ORPHANS );
 	OUTS( BREAK_ORPHANS );
-	OUTS( FOCUS_INSTALLED );
 	OUTS( YUM_OBSOLETES );
 #undef OUTS
+        os << "  focus	= "	<< _focus << endl;
 	os << "  distupgrade	= "	<< _distupgrade << endl;
         os << "  distupgrade_removeunsupported	= " << _distupgrade_removeunsupported << endl;
 	os << "  solveSrcPackages	= "	<< _solveSrcPackages << endl;
@@ -183,6 +289,7 @@ SATResolver::SATResolver (const ResPool & pool, sat::detail::CPool *satPool)
     : _pool(pool)
     , _satPool(satPool)
     , _satSolver(NULL)
+    , _focus			( ZConfig::instance().solver_focus() )
     , _fixsystem(false)
     , _allowdowngrade		( false )
     , _allownamechange		( true )	// bsc#1071466
@@ -357,24 +464,6 @@ class CheckIfUpdate : public resfilter::PoolItemFilterFunctor
 };
 
 
-class CollectPseudoInstalled : public resfilter::PoolItemFilterFunctor
-{
-  public:
-    Queue *solvableQueue;
-
-    CollectPseudoInstalled( Queue *queue )
-	:solvableQueue (queue)
-    {}
-
-    // collecting PseudoInstalled items
-    bool operator()( PoolItem item )
-    {
-      if ( traits::isPseudoInstalled( item.satSolvable().kind() ) )
-        queue_push( solvableQueue, item.satSolvable().id() );
-      return true;
-    }
-};
-
 bool
 SATResolver::solving(const CapabilitySet & requires_caps,
 		     const CapabilitySet & conflict_caps)
@@ -397,6 +486,7 @@ SATResolver::solving(const CapabilitySet & requires_caps,
 	queue_push( &(_jobQueue), SOLVER_DROP_ORPHANED|SOLVER_SOLVABLE_ALL);
 	queue_push( &(_jobQueue), 0 );
     }
+    solverSetFocus( *_satSolver, _focus );
     solver_set_flag(_satSolver, SOLVER_FLAG_ADD_ALREADY_RECOMMENDED, !_ignorealreadyrecommended);
     solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE,		_allowdowngrade);
     solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_NAMECHANGE,		_allownamechange);
@@ -411,20 +501,60 @@ SATResolver::solving(const CapabilitySet & requires_caps,
     solver_set_flag(_satSolver, SOLVER_FLAG_DUP_ALLOW_NAMECHANGE,	_dup_allownamechange );
     solver_set_flag(_satSolver, SOLVER_FLAG_DUP_ALLOW_ARCHCHANGE,	_dup_allowarchchange );
     solver_set_flag(_satSolver, SOLVER_FLAG_DUP_ALLOW_VENDORCHANGE,	_dup_allowvendorchange );
-#if 1
-#define HACKENV(X,D) solver_set_flag(_satSolver, X, env::HACKENV( #X, D ) );
-    HACKENV( SOLVER_FLAG_DUP_ALLOW_DOWNGRADE,	_dup_allowdowngrade );
-    HACKENV( SOLVER_FLAG_DUP_ALLOW_NAMECHANGE,	_dup_allownamechange );
-    HACKENV( SOLVER_FLAG_DUP_ALLOW_ARCHCHANGE,	_dup_allowarchchange );
-    HACKENV( SOLVER_FLAG_DUP_ALLOW_VENDORCHANGE,_dup_allowvendorchange );
-#undef HACKENV
-#endif
+
     sat::Pool::instance().prepare();
 
     // Solve !
     MIL << "Starting solving...." << endl;
     MIL << *this;
-    solver_solve( _satSolver, &(_jobQueue) );
+    if ( solver_solve( _satSolver, &(_jobQueue) ) == 0 )
+    {
+      // bsc#1155819: Weakremovers of future product not evaluated.
+      // Do a 2nd run to cleanup weakremovers() of to be installed
+      // Produtcs unless removeunsupported is active (cleans up all).
+      if ( _distupgrade )
+      {
+	if ( _distupgrade_removeunsupported )
+	  MIL << "Droplist processing not needed. RemoveUnsupported is On." << endl;
+	else if ( ! ZConfig::instance().solverUpgradeRemoveDroppedPackages() )
+	  MIL << "Droplist processing is disabled in ZConfig." << endl;
+	else
+	{
+	  bool resolve = false;
+	  MIL << "Checking droplists ..." << endl;
+	  // get Solvables to be installed...
+	  sat::SolvableQueue decisionq;
+	  solver_get_decisionqueue( _satSolver, decisionq );
+	  for ( sat::detail::IdType id : decisionq )
+	  {
+	    if ( id < 0 )
+	      continue;
+	    sat::Solvable slv { (sat::detail::SolvableIdType)id };
+	    // get product buddies (they carry the weakremover)...
+	    static const Capability productCap { "product()" };
+	    if ( slv && slv.provides().matches( productCap ) )
+	    {
+	      CapabilitySet droplist { slv.valuesOfNamespace( "weakremover" ) };
+	      MIL << "Droplist for " << slv << ": size " << droplist.size() << endl;
+	      if ( !droplist.empty() )
+	      {
+		for ( const auto & cap : droplist )
+		{
+		  queue_push( &_jobQueue, SOLVER_DROP_ORPHANED | SOLVER_SOLVABLE_NAME );
+		  queue_push( &_jobQueue, cap.id() );
+		}
+		// PIN product - a safety net to prevent cleanup from changing the decision for this product
+		queue_push( &(_jobQueue), SOLVER_INSTALL | SOLVER_SOLVABLE );
+		queue_push( &(_jobQueue), id );
+		resolve = true;
+	      }
+	    }
+	  }
+	  if ( resolve )
+	    solver_solve( _satSolver, &(_jobQueue) );
+	}
+      }
+    }
     MIL << "....Solver end" << endl;
 
     // copying solution back to zypp pool
@@ -438,8 +568,12 @@ SATResolver::solving(const CapabilitySet & requires_caps,
     solver_get_decisionqueue(_satSolver, &decisionq);
     for ( int i = 0; i < decisionq.count; ++i )
     {
-      sat::Solvable slv( decisionq.elements[i] );
-      if ( !slv || slv.isSystem() )
+      Id p = decisionq.elements[i];
+      if ( p < 0 )
+	continue;
+
+      sat::Solvable slv { (sat::detail::SolvableIdType)p };
+      if ( ! slv || slv.isSystem() )
 	continue;
 
       PoolItem poolItem( slv );
@@ -488,81 +622,10 @@ SATResolver::solving(const CapabilitySet & requires_caps,
       }
     }
 
-    Queue recommendations;
-    Queue suggestions;
-    Queue orphaned;
-    Queue unneeded;
-    queue_init(&recommendations);
-    queue_init(&suggestions);
-    queue_init(&orphaned);
-    queue_init(&unneeded);
-    solver_get_recommendations(_satSolver, &recommendations, &suggestions, 0);
-    solver_get_orphaned(_satSolver, &orphaned);
-    solver_get_unneeded(_satSolver, &unneeded, 1);
-    /*  solvables which are recommended */
-    for ( int i = 0; i < recommendations.count; ++i )
-    {
-      PoolItem poolItem( getPoolItem( recommendations.elements[i] ) );
-      poolItem.status().setRecommended( true );
-    }
-
-    /*  solvables which are suggested */
-    for ( int i = 0; i < suggestions.count; ++i )
-    {
-      PoolItem poolItem( getPoolItem( suggestions.elements[i] ) );
-      poolItem.status().setSuggested( true );
-    }
-
-    _problem_items.clear();
-    /*  solvables which are orphaned */
-    for ( int i = 0; i < orphaned.count; ++i )
-    {
-      PoolItem poolItem( getPoolItem( orphaned.elements[i] ) );
-      poolItem.status().setOrphaned( true );
-      _problem_items.push_back( poolItem );
-    }
-
-    /*  solvables which are unneeded */
-    for ( int i = 0; i < unneeded.count; ++i )
-    {
-      PoolItem poolItem( getPoolItem( unneeded.elements[i] ) );
-      poolItem.status().setUnneeded( true );
-    }
-
-    queue_free(&recommendations);
-    queue_free(&suggestions);
-    queue_free(&orphaned);
-    queue_free(&unneeded);
-
-    /* Write validation state back to pool */
-    Queue flags, solvableQueue;
-
-    queue_init(&flags);
-    queue_init(&solvableQueue);
-
-    CollectPseudoInstalled collectPseudoInstalled(&solvableQueue);
-    invokeOnEach( _pool.begin(),
-		  _pool.end(),
-		  functor::functorRef<bool,PoolItem> (collectPseudoInstalled) );
-    solver_trivial_installable(_satSolver, &solvableQueue, &flags );
-    for (int i = 0; i < solvableQueue.count; i++) {
-	PoolItem item = _pool.find (sat::Solvable(solvableQueue.elements[i]));
-	item.status().setUndetermined();
-
-	if (flags.elements[i] == -1) {
-	    item.status().setNonRelevant();
-	    XDEBUG("SATSolutionToPool(" << item << " ) nonRelevant !");
-	} else if (flags.elements[i] == 1) {
-	    item.status().setSatisfied();
-	    XDEBUG("SATSolutionToPool(" << item << " ) satisfied !");
-	} else if (flags.elements[i] == 0) {
-	    item.status().setBroken();
-	    XDEBUG("SATSolutionToPool(" << item << " ) broken !");
-	}
-    }
-    queue_free(&(solvableQueue));
-    queue_free(&flags);
-
+    // copy back computed status values to pool
+    // (on the fly cache orphaned items for the UI)
+    solverCopyBackWeak( *_satSolver, _problem_items );
+    solverCopyBackValidate( *_satSolver, _pool );
 
     // Solvables which were selected due requirements which have been made by the user will
     // be selected by APPL_LOW. We can't use any higher level, because this setting must
@@ -624,6 +687,14 @@ SATResolver::solverInit(const PoolItemList & weakItems)
         queue_push( &(_jobQueue), id );
     }
 
+    // Ad rules for retracted patches and packages
+    {
+      queue_push( &(_jobQueue), SOLVER_BLACKLIST|SOLVER_SOLVABLE_PROVIDES );
+      queue_push( &(_jobQueue), sat::Solvable::retractedToken.id() );
+      queue_push( &(_jobQueue), SOLVER_BLACKLIST|SOLVER_SOLVABLE_PROVIDES );
+      queue_push( &(_jobQueue), sat::Solvable::ptfToken.id() );
+    }
+
     // Ad rules for changed requestedLocales
     {
       const auto & trackedLocaleIds( myPool().trackedLocaleIds() );
@@ -650,38 +721,6 @@ SATResolver::solverInit(const PoolItemList & weakItems)
     }
 
     ::pool_add_userinstalled_jobs(_satPool, sat::Pool::instance().autoInstalled(), &(_jobQueue), GET_USERINSTALLED_NAMES|GET_USERINSTALLED_INVERTED);
-
-    if ( _distupgrade )
-    {
-      if ( ZConfig::instance().solverUpgradeRemoveDroppedPackages() )
-      {
-        MIL << "Checking droplists ..." << endl;
-        // Dropped packages: look for 'weakremover()' provides
-        // in dup candidates of installed products.
-        ResPoolProxy proxy( ResPool::instance().proxy() );
-        for_( it, proxy.byKindBegin<Product>(), proxy.byKindEnd<Product>() )
-        {
-          if ( (*it)->onSystem() ) // (to install) or (not to delete)
-          {
-            Product::constPtr prodCand( (*it)->candidateAsKind<Product>() );
-            if ( ! prodCand )
-              continue; // product no longer available
-
-            CapabilitySet droplist( prodCand->droplist() );
-            dumpRangeLine( MIL << "Droplist for " << (*it)->candidateObj() << ": " << droplist.size() << " ", droplist.begin(), droplist.end() ) << endl;
-            for_( cap, droplist.begin(), droplist.end() )
-            {
-              queue_push( &_jobQueue, SOLVER_DROP_ORPHANED | SOLVER_SOLVABLE_NAME );
-              queue_push( &_jobQueue, cap->id() );
-            }
-          }
-        }
-      }
-      else
-      {
-        MIL << "Droplist processing is disabled." << endl;
-      }
-    }
 }
 
 void
@@ -840,6 +879,7 @@ void SATResolver::doUpdate()
 	queue_push( &(_jobQueue), SOLVER_DROP_ORPHANED|SOLVER_SOLVABLE_ALL);
 	queue_push( &(_jobQueue), 0 );
     }
+    solverSetFocus( *_satSolver, _focus );
     solver_set_flag(_satSolver, SOLVER_FLAG_ADD_ALREADY_RECOMMENDED, !_ignorealreadyrecommended);
     solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE,		_allowdowngrade);
     solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_NAMECHANGE,		_allownamechange);
@@ -868,19 +908,15 @@ void SATResolver::doUpdate()
     solver_get_decisionqueue(_satSolver, &decisionq);
     for (int i = 0; i < decisionq.count; i++)
     {
-      Id p;
-      p = decisionq.elements[i];
-      if (p < 0 || !sat::Solvable(p))
-	continue;
-      if (sat::Solvable(p).repository().get() == _satSolver->pool->installed)
+      Id p = decisionq.elements[i];
+      if ( p < 0 )
 	continue;
 
-      PoolItem poolItem = _pool.find (sat::Solvable(p));
-      if (poolItem) {
-	  SATSolutionToPool (poolItem, ResStatus::toBeInstalled, ResStatus::SOLVER);
-      } else {
-	  ERR << "id " << p << " not found in ZYPP pool." << endl;
-      }
+      sat::Solvable solv { (sat::detail::SolvableIdType)p };
+      if ( ! solv || solv.isSystem() )
+	continue;
+
+      SATSolutionToPool( PoolItem(solv), ResStatus::toBeInstalled, ResStatus::SOLVER );
     }
     queue_free(&decisionq);
 
@@ -908,6 +944,12 @@ void SATResolver::doUpdate()
 	  ERR << "id " << i << " not found in ZYPP pool." << endl;
       }
     }
+
+    // copy back computed status values to pool
+    // (on the fly cache orphaned items for the UI)
+    solverCopyBackWeak( *_satSolver, _problem_items );
+    solverCopyBackValidate( *_satSolver, _pool );
+
     MIL << "SATResolver::doUpdate() done" << endl;
 }
 
@@ -1001,7 +1043,7 @@ std::vector<std::string> SATResolver::SATgetCompleteProblemInfoStrings ( Id prob
   return ret;
 }
 
-string SATResolver::SATprobleminfoString(Id problem, string &detail, Id &ignoreId)
+std::string SATResolver::SATprobleminfoString(Id problem, std::string &detail, Id &ignoreId)
 {
   // FIXME: solver_findallproblemrules to get all rules for this problem
   // (the 'most relevabt' one returned by solver_findproblemrule is embedded
@@ -1011,7 +1053,7 @@ string SATResolver::SATprobleminfoString(Id problem, string &detail, Id &ignoreI
 
 std::string SATResolver::SATproblemRuleInfoString (Id probr, std::string &detail, Id &ignoreId)
 {
-  string ret;
+  std::string ret;
   sat::detail::CPool *pool = _satSolver->pool;
   Id dep, source, target;
   SolverRuleinfo type = solver_ruleinfo(_satSolver, probr, &source, &target, &dep);
@@ -1083,7 +1125,7 @@ std::string SATResolver::SATproblemRuleInfoString (Id probr, std::string &detail
 	  sat::WhatProvides possibleProviders(cap);
 
 	  // check, if a provider will be deleted
-	  typedef list<PoolItem> ProviderList;
+	  typedef std::list<PoolItem> ProviderList;
 	  ProviderList providerlistInstalled, providerlistUninstalled;
 	  for_( iter1, possibleProviders.begin(), possibleProviders.end() ) {
 	      PoolItem provider1 = ResPool::instance().find( *iter1 );
@@ -1159,9 +1201,9 @@ SATResolver::problems ()
 	while ((problem = solver_next_problem(_satSolver, problem)) != 0) {
 	    MIL << "Problem " <<  pcnt++ << ":" << endl;
 	    MIL << "====================================" << endl;
-	    string detail;
+	    std::string detail;
 	    Id ignoreId;
-	    string whatString = SATprobleminfoString (problem,detail,ignoreId);
+	    std::string whatString = SATprobleminfoString (problem,detail,ignoreId);
 	    MIL << whatString << endl;
 	    MIL << "------------------------------------" << endl;
             ResolverProblem_Ptr resolverProblem = new ResolverProblem (whatString, detail, SATgetCompleteProblemInfoStrings( problem ));
@@ -1182,12 +1224,12 @@ SATResolver::problems ()
 				if (poolItem) {
 				    if (pool->installed && s.get()->repo == pool->installed) {
 					problemSolution->addSingleAction (poolItem, REMOVE);
-					string description = str::form (_("remove lock to allow removal of %s"),  s.asString().c_str() );
+					std::string description = str::form (_("remove lock to allow removal of %s"),  s.asString().c_str() );
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    } else {
 					problemSolution->addSingleAction (poolItem, KEEP);
-					string description = str::form (_("do not install %s"), s.asString().c_str());
+					std::string description = str::form (_("do not install %s"), s.asString().c_str());
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    }
@@ -1202,12 +1244,12 @@ SATResolver::problems ()
 				if (poolItem) {
 				    if (pool->installed && s.get()->repo == pool->installed) {
 					problemSolution->addSingleAction (poolItem, KEEP);
-					string description = str::form (_("keep %s"), s.asString().c_str());
+					std::string description = str::form (_("keep %s"), s.asString().c_str());
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    } else {
 					problemSolution->addSingleAction (poolItem, UNLOCK);
-					string description = str::form (_("remove lock to allow installation of %s"), itemToString( poolItem ).c_str());
+					std::string description = str::form (_("remove lock to allow installation of %s"), itemToString( poolItem ).c_str());
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    }
@@ -1223,7 +1265,7 @@ SATResolver::problems ()
 				    new SolverQueueItemInstall(_pool, ident.asString(), false );
 				problemSolution->addSingleAction (install, REMOVE_SOLVE_QUEUE_ITEM);
 
-				string description = str::form (_("do not install %s"), ident.c_str() );
+				std::string description = str::form (_("do not install %s"), ident.c_str() );
 				MIL << description << endl;
 				problemSolution->addDescription (description);
 				}
@@ -1244,7 +1286,7 @@ SATResolver::problems ()
 				    new SolverQueueItemDelete(_pool, ident.asString(), false );
 				problemSolution->addSingleAction (del, REMOVE_SOLVE_QUEUE_ITEM);
 
-				string description = str::form (_("keep %s"), ident.c_str());
+				std::string description = str::form (_("keep %s"), ident.c_str());
 				MIL << description << endl;
 				problemSolution->addDescription (description);
 				}
@@ -1252,7 +1294,7 @@ SATResolver::problems ()
 			    case SOLVER_INSTALL | SOLVER_SOLVABLE_PROVIDES:
 				{
 				problemSolution->addSingleAction (Capability(what), REMOVE_EXTRA_REQUIRE);
-				string description = "";
+				std::string description = "";
 
 				// Checking if this problem solution would break your system
 				if (system_requires.find(Capability(what)) != system_requires.end()) {
@@ -1260,7 +1302,7 @@ SATResolver::problems ()
 				    resolverProblem->setDetails( resolverProblem->description() + "\n" + resolverProblem->details() );
 				    resolverProblem->setDescription(_("This request will break your system!"));
 				    description = _("ignore the warning of a broken system");
-                                    description += string(" (requires:")+pool_dep2str(pool, what)+")";
+				    description += std::string(" (requires:")+pool_dep2str(pool, what)+")";
                                     MIL << description << endl;
                                     problemSolution->addFrontDescription (description);
 				} else {
@@ -1273,7 +1315,7 @@ SATResolver::problems ()
 			    case SOLVER_ERASE | SOLVER_SOLVABLE_PROVIDES:
 				{
 				problemSolution->addSingleAction (Capability(what), REMOVE_EXTRA_CONFLICT);
-				string description = "";
+				std::string description = "";
 
 				// Checking if this problem solution would break your system
 				if (system_conflicts.find(Capability(what)) != system_conflicts.end()) {
@@ -1281,7 +1323,7 @@ SATResolver::problems ()
 				    resolverProblem->setDetails( resolverProblem->description() + "\n" + resolverProblem->details() );
 				    resolverProblem->setDescription(_("This request will break your system!"));
 				    description = _("ignore the warning of a broken system");
-                                    description += string(" (conflicts:")+pool_dep2str(pool, what)+")";
+				    description += std::string(" (conflicts:")+pool_dep2str(pool, what)+")";
                                     MIL << description << endl;
                                     problemSolution->addFrontDescription (description);
 
@@ -1299,7 +1341,7 @@ SATResolver::problems ()
 				if (poolItem) {
 				    if (pool->installed && s.get()->repo == pool->installed) {
 					problemSolution->addSingleAction (poolItem, KEEP);
-					string description = str::form (_("do not install most recent version of %s"), s.asString().c_str());
+					std::string description = str::form (_("do not install most recent version of %s"), s.asString().c_str());
 					MIL << description << endl;
 					problemSolution->addDescription (description);
 				    } else {
@@ -1320,12 +1362,12 @@ SATResolver::problems ()
 			PoolItem poolItem = _pool.find (s);
 			if (pool->installed && s.get()->repo == pool->installed) {
 			    problemSolution->addSingleAction (poolItem, LOCK);
-			    string description = str::form (_("keep %s despite the inferior architecture"), s.asString().c_str());
+			    std::string description = str::form (_("keep %s despite the inferior architecture"), s.asString().c_str());
 			    MIL << description << endl;
 			    problemSolution->addDescription (description);
 			} else {
 			    problemSolution->addSingleAction (poolItem, INSTALL);
-			    string description = str::form (_("install %s despite the inferior architecture"), s.asString().c_str());
+			    std::string description = str::form (_("install %s despite the inferior architecture"), s.asString().c_str());
 			    MIL << description << endl;
 			    problemSolution->addDescription (description);
 			}
@@ -1334,16 +1376,36 @@ SATResolver::problems ()
 			PoolItem poolItem = _pool.find (s);
 			if (pool->installed && s.get()->repo == pool->installed) {
 			    problemSolution->addSingleAction (poolItem, LOCK);
-			    string description = str::form (_("keep obsolete %s"), s.asString().c_str());
+			    std::string description = str::form (_("keep obsolete %s"), s.asString().c_str());
 			    MIL << description << endl;
 			    problemSolution->addDescription (description);
 			} else {
 			    problemSolution->addSingleAction (poolItem, INSTALL);
-			    string description = str::form (_("install %s from excluded repository"), s.asString().c_str());
+			    std::string description = str::form (_("install %s from excluded repository"), s.asString().c_str());
 			    MIL << description << endl;
 			    problemSolution->addDescription (description);
 			}
-		    } else {
+		    } else if ( p == SOLVER_SOLUTION_BLACK ) {
+			// Allow to install a blacklisted package (PTF, retracted,...).
+			// For not-installed items only
+			s = mapSolvable (rp);
+			PoolItem poolItem = _pool.find (s);
+
+			problemSolution->addSingleAction (poolItem, INSTALL);
+			std::string description;
+			if ( s.isRetracted() ) {
+			  // translator: %1% is a package name
+			  description = str::Format(_("install %1% although it has been retracted")) % s.asString();
+			} else if ( s.isPtf() ) {
+			  // translator: %1% is a package name
+			  description = str::Format(_("allow to install the PTF %1%")) % s.asString();
+			} else {
+			  // translator: %1% is a package name
+			  description = str::Format(_("install %1% although it is blacklisted")) % s.asString();
+			}
+			MIL << description << endl;
+			problemSolution->addDescription( description );
+		    } else if ( p > 0 ) {
 			/* policy, replace p with rp */
 			s = mapSolvable (p);
 			PoolItem itemFrom = _pool.find (s);
@@ -1359,14 +1421,14 @@ SATResolver::problems ()
 
 				if ((illegal & POLICY_ILLEGAL_DOWNGRADE) != 0)
 				{
-				    string description = str::form (_("downgrade of %s to %s"), s.asString().c_str(), sd.asString().c_str());
+				    std::string description = str::form (_("downgrade of %s to %s"), s.asString().c_str(), sd.asString().c_str());
 				    MIL << description << endl;
 				    problemSolution->addDescription (description);
 				    gotone = 1;
 				}
 				if ((illegal & POLICY_ILLEGAL_ARCHCHANGE) != 0)
 				{
-				    string description = str::form (_("architecture change of %s to %s"), s.asString().c_str(), sd.asString().c_str());
+				    std::string description = str::form (_("architecture change of %s to %s"), s.asString().c_str(), sd.asString().c_str());
 				    MIL << description << endl;
 				    problemSolution->addDescription (description);
 				    gotone = 1;
@@ -1375,7 +1437,7 @@ SATResolver::problems ()
 				{
                                     IdString s_vendor( s.vendor() );
                                     IdString sd_vendor( sd.vendor() );
-				    string description = str::form (_("install %s (with vendor change)\n  %s  -->  %s") ,
+				    std::string description = str::form (_("install %s (with vendor change)\n  %s  -->  %s") ,
 								    sd.asString().c_str(),
                                                                     ( s_vendor ? s_vendor.c_str() : " (no vendor) " ),
                                                                     ( sd_vendor ? sd_vendor.c_str() : " (no vendor) " ) );
@@ -1384,7 +1446,7 @@ SATResolver::problems ()
 				    gotone = 1;
 				}
 				if (!gotone) {
-				    string description = str::form (_("replacement of %s with %s"), s.asString().c_str(), sd.asString().c_str());
+				    std::string description = str::form (_("replacement of %s with %s"), s.asString().c_str(), sd.asString().c_str());
 				    MIL << description << endl;
 				    problemSolution->addDescription (description);
 				}
@@ -1395,13 +1457,18 @@ SATResolver::problems ()
 			else
 			{
 			    if (itemFrom) {
-				string description = str::form (_("deinstallation of %s"), s.asString().c_str());
+				std::string description = str::form (_("deinstallation of %s"), s.asString().c_str());
 				MIL << description << endl;
 				problemSolution->addDescription (description);
 				problemSolution->addSingleAction (itemFrom, REMOVE);
 			    }
 			}
 		    }
+		    else
+		    {
+		      INT << "Unknown solution " << p << endl;
+		    }
+
 		}
 		resolverProblem->addSolution (problemSolution,
 					      problemSolution->actionCount() > 1 ? true : false); // Solutions with more than 1 action will be shown first.
