@@ -747,6 +747,7 @@ namespace zypp
     , _requestedLocalesFile( home() / "RequestedLocales" )
     , _autoInstalledFile( home() / "AutoInstalled" )
     , _hardLocksFile( Pathname::assertprefix( _root, ZConfig::instance().locksFile() ) )
+    , _vendorAttr( Pathname::assertprefix( _root, ZConfig::instance().vendorPath() ) )
     {
       _rpm.initDatabase( root_r, doRebuild_r );
 
@@ -1958,6 +1959,13 @@ namespace zypp
 
     ///////////////////////////////////////////////////////////////////
 
+    void TargetImpl::vendorAttr( VendorAttr vendorAttr_r )
+    {
+      MIL << "New VendorAttr: " << vendorAttr_r << endl;
+      _vendorAttr = std::move(vendorAttr_r);
+    }
+    ///////////////////////////////////////////////////////////////////
+
     void TargetImpl::installSrcPackage( const SrcPackage_constPtr & srcPackage_r )
     {
       // provide on local disk
